﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Xml;
using System.Xml.Xsl;
using System.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Xml.Serialization;

namespace CopyAzureStorage
{
    // This contains all the helpers used by events on the form.
    public partial class VHDandVMHelper
    {
        // Update this to Read and Write from other containers.
        // Will also need to update the .vhd validation on the Destination blob text field.
        private String containerName = "vhds";

        // Used to know if the form is used a 2nd time without ending since the Security Policy will already be setup
        private Boolean hasSecurityToken = false;
        private String globalSecurityToken = String.Empty;

        // Used to store if RestApi is enabled
        private Boolean hasRESTApiEnabled = false;
        private String globalAzureSourceAccountID = String.Empty;
        private String certificatePath = String.Empty;

        // Used to make these items global
        private Disks sourceDisks = null;
        private Images sourceImages = null;
        private Boolean hasSourceDisks = false;
        private Boolean hasSourceImages = false;

        // Used to track total download and upload times
        private DateTime startDownload = new DateTime();
        private DateTime endDownload = new DateTime();
        private DateTime startUpload = new DateTime();
        private DateTime endUpload = new DateTime();

        // Define the global variables for REST API
        public String postBodyString = string.Empty;
        private String apiVersion = "2012-03-01";
        
        internal void UpdateSourceVHDListBox()
        {
            bool hasSourceStorage = true;
            dgSourceStore.Rows.Clear();

            txtMain.Text += "Reading List of blobs in the Source Account \r\n";

            CloudStorageAccount sourceStorageAccount = null;

            try
            {
                sourceStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("SourceStorageConnection"));
            }
            catch (Exception ex)
            {
                hasSourceStorage = false;
            }

            if (hasSourceStorage)
            {

                //CloudStorageAccount sourceStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("SourceStorageConnection"));
                CloudBlobClient sourceBlobClient = sourceStorageAccount.CreateCloudBlobClient();

                labelSourceTxt.Text = sourceStorageAccount.Credentials.AccountName;

                txtMain.Text += "Reading from Blob Endpoint " + sourceStorageAccount.BlobEndpoint + "\r\n";

                // Retrieve reference to a previously created container
                CloudBlobContainer sourceContainer = sourceBlobClient.GetContainerReference(containerName);

                // Created the container if it does not exist
                CreateContainerIfNeeded(sourceContainer);

                var vhdsBlobList = sourceContainer.ListBlobs(true, BlobListingDetails.Copy);

                txtMain.Text += "Reading from Container " + containerName + "\r\n";

                foreach (var vhd in vhdsBlobList)
                {
                    var singeVHD = vhd as CloudBlob;

                    string vhdName = singeVHD.Name;

                    string[] dataRow = { vhdName, singeVHD.Properties.LeaseState.ToString(), singeVHD.Properties.Length / 1048576 + " MB" };

                    // Add the Disk to the Data Grid
                    dgSourceStore.Rows.Add(dataRow);

                    txtMain.Text += "Reading Source Blob Named " + singeVHD.Name + "\r\n";
                }

                txtMain.Text += "All blobs in the Container have been read." + "\r\n";

                // Remove any items that is selected
                dgSourceStore.ClearSelection();
            }
        }

        internal void UpdateDestinationVHDListBox()
        {
            // Clear the Data Grid
            dgDestinationStore.Rows.Clear();
            
            // Check if this is Source Only and not check for anything for the Destination
            if (checkBoxSourceOnly.Checked)
                return;

            txtMain.Text += "Reading List of blobs in the Destination Account \r\n";

            CloudStorageAccount destinationStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("DestinationStorageConnection"));
            CloudBlobClient destinationBlobClient = destinationStorageAccount.CreateCloudBlobClient();

            labelDestinationTxt.Text = destinationStorageAccount.Credentials.AccountName;

            txtMain.Text += "Reading from Blob Endpoint " + destinationStorageAccount.BlobEndpoint + "\r\n";

            // Retrieve reference to a previously created container
            CloudBlobContainer destinationContainer = destinationBlobClient.GetContainerReference(containerName);

            // Created the container if it does not exist
            CreateContainerIfNeeded(destinationContainer);

            var vhdsBlobList = destinationContainer.ListBlobs(true, BlobListingDetails.Copy);

            txtMain.Text += "Reading from Container " + containerName + "\r\n";

            foreach (var vhd in vhdsBlobList)
            {
                var singeVHD = vhd as CloudBlob;

                string vhdName = singeVHD.Name;

                string[] dataRow = { vhdName, singeVHD.Properties.LeaseState.ToString(), singeVHD.Properties.Length / 1048576 + " MB" };

                // Add the Disk to the Data Grid
                dgDestinationStore.Rows.Add(dataRow); 
                
                txtMain.Text += "Reading Destination Blob Named " + singeVHD.Name + "\r\n";
            }

            txtMain.Text += "All blobs in the Container have been read." + "\r\n";

            // Remove any items that is selected
            dgDestinationStore.ClearSelection();
        }

        // This is the method that copies the selected VHD
        // Account details are read from the App.config file.
        // Source and Destination VHD are inputs on the Form.
        // Container name is hard coded to vhds.
        // If you want to copy to/from other containers just change this and remove the text validation on the Destination Blob Name.
        internal void CopyVHD()
        {
            // Get the Selected VHD Name
            DataGridViewRow singleRow = dgSourceStore.CurrentRow;
            string sourceVHDName = singleRow.Cells["cSourceVHDName"].Value.ToString();

            // Reads Configuration details from the App.config file.  Needs a reference to Microsoft.WindowsAzure.Configuration.dll.
            // Watch out of issues when using Copy and Paste from the portal.  I pasted into Notepad first and that seemed to mess something up.
            CloudStorageAccount sourceStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("SourceStorageConnection"));
            CloudStorageAccount destinationStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("DestinationStorageConnection"));

            txtMain.Text += "Source and Destination connection strings parse.\r\n";

            // Create the blob client using the Accounts above
            CloudBlobClient sourceBlobClient = sourceStorageAccount.CreateCloudBlobClient();
            CloudBlobClient destinationBlobClient = destinationStorageAccount.CreateCloudBlobClient();

            // Retrieve reference to a previously created container
            // Rename "vhds" as needed.  Can be used to read from any container.
            CloudBlobContainer sourceContainer = sourceBlobClient.GetContainerReference(containerName);
            CloudBlob sourceBlob = sourceContainer.GetBlobReference(sourceVHDName);

            txtMain.Text += "Getting Source Blog Named " + sourceVHDName;

            CloudBlobContainer destinationContainer = destinationBlobClient.GetContainerReference(containerName);

            // Created the container if it does not exist
            CreateContainerIfNeeded(destinationContainer);

            CloudBlob destinationBlob = destinationContainer.GetBlobReference(txtNewVHDName.Text);
            txtMain.Text += "Getting Destination blog reference to " + txtNewVHDName.Text + "\r\n";

            // Logic added to check if a security token has already been created
            if (!hasSecurityToken)
            {
                // Create a permission policy, consisting of a container-level access policy and a public access setting, and store it on the container. 
                // Hope to have a better approach soon.
                BlobContainerPermissions destinationContainerPermissions = new BlobContainerPermissions();

                // The container-level access policy provides read/write access to the container for 1 day.
                destinationContainerPermissions.SharedAccessPolicies.Add("imageCopyPolicy", new SharedAccessBlobPolicy()
                {
                    SharedAccessExpiryTime = DateTime.UtcNow.AddDays(1),
                    Permissions = SharedAccessBlobPermissions.Write | SharedAccessBlobPermissions.Read
                });

                destinationContainerPermissions.PublicAccess = BlobContainerPublicAccessType.Off;
                sourceContainer.SetPermissions(destinationContainerPermissions);

                // This gets the token needed for cross-account copies.  This is not needed to copy into the same account.
                globalSecurityToken = sourceContainer.GetSharedAccessSignature(new SharedAccessBlobPolicy(), "imageCopyPolicy");

                // Set the Global Variable
                hasSecurityToken = true;
            }

            txtMain.Text += "Added imageCopyPolicy shared access blog policy to Destination and returned Source Key\r\n";
            txtMain.Text += "Starting BLOG copy.\r\n";

            // This does the copy using the StartCopyFromBlog method in the 1.7.1 SDK.            
            string copyID = destinationBlob.StartCopyFromBlob(new Uri(sourceBlob.Uri.AbsoluteUri + globalSecurityToken));

            labelToDo.Text = "Request started.  If cross datacenter this will take a while.\r\nThis must complete before you can continue.";
            this.Refresh();

            MonitorCopy(destinationContainer);

            btnCopyStatus.Enabled = true;

            txtMain.Text += "Copy request complete.\r\n";
            labelToDo.Text = "Request complete.\r\nClose tool and update connection strings to run again.";
            this.Refresh();
        }

        // Do this in another thread?
        public void MonitorCopy(CloudBlobContainer destContainer)
        {
            bool pendingCopy = true;

            while (pendingCopy)
            {
                pendingCopy = false;

                // Update to get the one blog only
                var destBlobList = destContainer.ListBlobs(true, BlobListingDetails.Copy);

                //CloudBlob newBlob = destContainer.GetBlobReference(txtNewVHDName.Text).;

                foreach (var dest in destBlobList)
                {
                    if (dest.Uri.ToString().Contains(txtNewVHDName.Text))
                    {
                        var destBlob = dest as CloudBlob;

                        if (destBlob.CopyState.Status == CopyStatus.Aborted ||
                            destBlob.CopyState.Status == CopyStatus.Failed)
                        {
                            // Log the copy status description for diagnostics and restart copy
                            pendingCopy = true;
                            destBlob.StartCopyFromBlob(destBlob.CopyState.Source);
                        }
                        else if (destBlob.CopyState.Status == CopyStatus.Pending)
                        {
                            pendingCopy = true;
                        }
                        // else we completed this pending copy
                    }
                }

                Thread.Sleep(1000);
            };
        }

        // Used to create the container if it does not exist in the source or destination
        private void CreateContainerIfNeeded(CloudBlobContainer currentContainer)
        {
            try
            {
                // Approach per http://blog.smarx.com/posts/testing-existence-of-a-windows-azure-blob
                currentContainer.FetchAttributes();

                // If we get this far we know the container exists.
                txtMain.Text += "Container already exists.\r\n";
            }
            catch (StorageClientException e)
            {
                // Assume we got an exception because the container did not exist so we will try to create it
                currentContainer.CreateIfNotExist();
                txtMain.Text += "Container created.\r\n";
            }
        }
        
        private void UpdateRESTApiDetails()
        {
            string diskURL = "https://management.core.windows.net/" + globalAzureSourceAccountID + "/services/disks";
            GetRestAPIInfo(diskURL, "GET", "disks");

            if (hasSourceDisks)
            {
                dgAllDisks.Rows.Clear();
                dgVMList.Rows.Clear();

                // Populate the Disks Data Grid
                foreach (DisksDisk myDisk in sourceDisks.Items)
                {
                    DisksDiskAttachedTo[] myAttached = null;

                    string virtualMachineName = "NONE";
                    string serviceName = "NONE";
                    string myDiskText = String.Empty;
                    string deploymentName = "NONE";

                    if (myDisk.AttachedTo != null)
                    {
                        // Check if the disk is attached to a Virtual Machine
                        if (myDisk.AttachedTo.Count() > 0)
                        {
                            myAttached = myDisk.AttachedTo;
                        }

                        // Set the Hosted Service Name
                        virtualMachineName = myAttached[0].RoleName;

                        // Set the Hosted Service Name
                        serviceName = myAttached[0].HostedServiceName;

                        // Set the Deployment Name
                        deploymentName = myAttached[0].DeploymentName;
                    }

                    // TODO - VM's that have VHD in Broken Status are showing up and shouldn't
                    // Add the VM to the Data Grid
                    if (AddToDataGrid(virtualMachineName) && virtualMachineName.ToUpper() != "NONE")
                    {
                        string[] dataVMRow = { "false", serviceName, deploymentName, virtualMachineName, myDisk.Location };

                        // Add the VM to the Data Grid
                        dgVMList.Rows.Add(dataVMRow);
                    }

                    string[] dataRow = { myDisk.Name, myDisk.Location, serviceName, virtualMachineName };
                    
                    // Add the Disk to the Data Grid
                    dgAllDisks.Rows.Add(dataRow);
                }
            }

            string imageURL = "https://management.core.windows.net/" + globalAzureSourceAccountID + "/services/images";
            GetRestAPIInfo(imageURL, "GET", "images");

            if (hasSourceImages)
            {
                dgImages.Rows.Clear();

                // Populate the Images Listbox
                foreach (ImagesOSImage myImage in sourceImages.Items)
                {
                    // Images will return system images too - we only want to show custom images
                    if (myImage.Category.ToUpper() == "USER")
                    {
                        string[] dataRow = { myImage.Name, myImage.Location, myImage.OS };

                        // Add the Disk to the Data Grid
                        dgImages.Rows.Add(dataRow);
                    }
                }
            }
        }

        // Helper method to search the data grid to see if the VM name already exists
        private bool AddToDataGrid(String virtualMachineName)
        {
            // If no rows, add it
            if (dgVMList.Rows.Count < 1)
            {
                return true;
            }
            
            // Loop through all the rows and look for the VM name
            foreach (DataGridViewRow singleRow in dgVMList.Rows)
            {
                if (singleRow.Cells["cVMName"].Value.ToString().ToUpper() == virtualMachineName.ToUpper())
                {
                    return false;
                }
            }

            return true;
        }

        // Logic for REST API Service Call
        // Approach per MSND: http://msdn.microsoft.com/en-us/library/windowsazure/ee460782.aspx
        private string GetRestAPIInfo(string restURL, string method, string operation, RestAPICallRequest singleVM = null)
        {
            // If an Operation Call get the response status
            string operationsResponse = string.Empty;
            
            try
            {
                // Get the URL from the form.  Should be something like https://management.core.windows.net/<subscription-id>/services/<operation-type>
                Uri requestUri = new Uri(restURL);

                // Create the request and specify attributes of the request.
                HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

                // Define the required headers to specify the API version and operation type.
                request.Headers.Add("x-ms-version", apiVersion);
                request.Method = method;
                request.ContentType = "application/xml";

                // If POST set the Body
                if (method == "POST")
                {
                    XmlDocument xDoc = new XmlDocument();
                    xDoc.LoadXml(postBodyString);

                    Stream requestStream = request.GetRequestStream();
                    StreamWriter streamWriter = new StreamWriter(requestStream, System.Text.UTF8Encoding.UTF8);
                    xDoc.Save(streamWriter);

                    streamWriter.Close();
                    requestStream.Close();
                }

                // Get the Certificate
                X509Certificate2 myCert = new X509Certificate2(certificatePath);

                // Add the Certificate
                // Could replace with reading from the Certificate Store but I have issues with that
                request.ClientCertificates.Add(myCert);
                
                // Make the call using the web request.
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                HttpStatusCode responseStatus = response.StatusCode;

                switch (operation)
                {
                    case "images":
                        try
                        {
                            XmlSerializer ser = new XmlSerializer(typeof(Images));
                            sourceImages = (Images)ser.Deserialize(response.GetResponseStream());
                            txtMain.Text += "Parsing Images.\r\n";

                            if (sourceImages.Items.Count() > 0)
                                hasSourceImages = true;
                        }
                        catch (Exception)
                        {
                            txtMain.Text += "Error parsing Images.\r\n";
                        }
                        break;

                    case "disks":
                        try
                        {
                            XmlSerializer ser = new XmlSerializer(typeof(Disks));
                            sourceDisks = (Disks)ser.Deserialize(response.GetResponseStream());
                            txtMain.Text += "Parsing Disks.\r\n";

                            if (sourceDisks.Items.Count() > 0)
                                hasSourceDisks = true;
                        }
                        catch (Exception)
                        {
                            txtMain.Text += "Error parsing Disks.\r\n";
                        }
                        break;
                    
                    case "getvmdetails":
                        if (response.ContentLength > 0)
                        {

                            // Parse the web response.
                            Stream responseStream = response.GetResponseStream();
                            StreamReader reader = new StreamReader(responseStream);

                            XmlDocument xDocResp = new XmlDocument();
                            xDocResp.Load(reader);

                            // Cleanup
                            responseStream.Close();
                            reader.Close();
                        }
                        break;
                    case "getvmdetailsexport":
                        if (response.ContentLength > 0)
                        {
                            // Parse the web response.
                            Stream responseStream = response.GetResponseStream();
                            StreamReader reader = new StreamReader(responseStream);

                            XmlDocument xDocResp = new XmlDocument();
                            xDocResp.Load(reader);

                            // Save the RAW output from the API
                            xDocResp.Save(localVMStorageLocation + singleVM.ServiceName + "_" + singleVM.VMName + "_" + singleVM.Location + "_BACKUP.xml");

                            // Show the output
                            vmExportSize = xDocResp.GetElementsByTagName("RoleSize")[0].InnerText;

                            string outputFileName = localVMStorageLocation + singleVM.ServiceName + "_" + singleVM.VMName + "_" + singleVM.Location + "_" + vmExportSize + ".xml";
                            string outputFileNameTemp = localVMStorageLocation + singleVM.ServiceName + "_" + singleVM.VMName + "_" + singleVM.Location + "_" + vmExportSize + "_TEMP.xml";

                            // XSLT Transformation
                            // Create a writer for writing the transformed file.
                            XmlWriter writertemp = XmlWriter.Create(outputFileNameTemp);
                            XsltSettings settings = new XsltSettings();
                            settings.EnableScript = true;
                            
                            // Create and load the transform with script execution enabled.
                            XslCompiledTransform transformtemp = new XslCompiledTransform();
                            transformtemp.Load(@"XSLT\GetVM_to_CreateVM.xslt", settings, null);
                            
                            // Execute the transformation.
                            transformtemp.Transform(xDocResp, writertemp);

                            writertemp.Flush();
                            writertemp.Close();

                            XmlWriter writer = XmlWriter.Create(outputFileName);
                            XmlReader readertemp = XmlReader.Create(outputFileNameTemp);                            

                            XslCompiledTransform transform = new XslCompiledTransform();
                            transform.Load(@"XSLT\RemoveEmptyNodes.xslt", settings, null);

                            // Execute the transformation.
                            transform.Transform(readertemp, writer);

                            writer.Flush();
                            writer.Close();

                            // Delete the Temp file used in the XSLT
                            File.Delete(outputFileNameTemp);

                            // Cleanup
                            responseStream.Close();
                            reader.Close();
                        }
                        break;
                    default:
                        // If the response is NULL nothing to show
                        if (response.ContentLength > 0)
                        {
                            // Parse the web response.
                            Stream responseStream = response.GetResponseStream();
                            StreamReader reader = new StreamReader(responseStream);

                            XmlDocument xDocResp = new XmlDocument();
                            xDocResp.Load(reader);

                            // Check the response to see if it is an Operation Message - if so get the Status
                            if (restURL.Contains("/operations/"))
                            {
                                operationsResponse = xDocResp.SelectSingleNode("/*[local-name()='Operation' and namespace-uri()='http://schemas.microsoft.com/windowsazure']/*[local-name()='Status' and namespace-uri()='http://schemas.microsoft.com/windowsazure']").InnerXml;
                            }

                            // Cleanup
                            responseStream.Close();
                            reader.Close();
                        }
                        break;
                }

                // Get the Request ID and set the URL to check the status if this is not an Operations Check and a 202 response for Async Request
                if (!restURL.Contains("/operations/") && responseStatus == HttpStatusCode.Accepted)
                {
                    WebHeaderCollection responseHeaders = response.Headers;
                    operationsResponse = responseHeaders.Get("x-ms-request-id");
                    //txtMain.Text = "202 - Accepted returned.  URL Populated with value to check Async result.";
                }

                // CLose the HttpWebResponse
                response.Close();
            }
            catch (WebException exWeb)
            {
                if (exWeb.Response.ContentLength > 0)
                {
                    // Parse the web response.
                    Stream responseStream = exWeb.Response.GetResponseStream();
                    StreamReader reader = new StreamReader(responseStream);

                    XmlDocument xDocResp = new XmlDocument();
                    xDocResp.Load(reader);
                    HttpWebResponse responseStatus = (HttpWebResponse)exWeb.Response;
                    operationsResponse = "Web Error making REST API call.\r\nMessage: " + exWeb.Message + "\r\nResponse:\r\n" + NiceFormatXml(xDocResp);

                    responseStream.Close();
                    reader.Close();
                }

                throw exWeb;
            }
            catch (Exception ex)
            {
                operationsResponse = "Error making REST API call.\r\n" + ex.Message + "\r\n";

                throw ex;
            }

            return operationsResponse;
        }


        private string NiceFormatXml(XmlDocument xDoc)
        {
            StringBuilder niceString = new StringBuilder();
            StringWriter strWriter = new StringWriter(niceString);
            XmlTextWriter xmlWriter = new XmlTextWriter(strWriter);

            xmlWriter.Formatting = Formatting.Indented;
            xDoc.WriteTo(xmlWriter);

            xmlWriter.Close();
            strWriter.Close();

            return niceString.ToString();
        }
    
        ////This tool was originally designed for cross account Azure copies.  
        ////Providing very basic upload and download support.  Other tools do it better.
        ////This will block the current thread and take a long time to complete.
        //private void UploadVHDToSource(string UploadFileName)
        //{
        //    txtMain.Text += "Starting Upload of VHD.  This will take a while.\r\n";
        //    startUpload = DateTime.Now;

        //    CloudStorageAccount sourceStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("SourceStorageConnection"));
        //    CloudBlobClient sourceBlobClient = sourceStorageAccount.CreateCloudBlobClient();

        //    // Retrieve reference to a previously created container
        //    CloudBlobContainer sourceContainer = sourceBlobClient.GetContainerReference(containerName);

        //    // Created the container if it does not exist
        //    CreateContainerIfNeeded(sourceContainer);

        //    // Get the File Name of the VHD.
        //    // Currently does not support renaming the uploaded file
        //    CloudBlockBlob sourceBlob = sourceContainer.GetBlockBlobReference(Path.GetFileName(UploadFileName));

        //    // Set a new timeout for the huge download
        //    BlobRequestOptions setOptions = new BlobRequestOptions();
        //    setOptions.Timeout = new TimeSpan(10, 0, 0);

        //    // Upload the blob to a file stream.
        //    FileStream stream = new FileStream(UploadFileName, FileMode.Append);
        //    sourceBlob.BeginUploadFromStream(stream, UploadBlobToStreamCallback, new Object[] { sourceBlob, stream });
        //}

        //private void UploadBlobToStreamCallback(IAsyncResult result)
        //{
        //    // Get array passed to callback.
        //    Object[] state = (Object[])result.AsyncState;

        //    CloudPageBlob sourceBlob = (CloudPageBlob)state[0];
        //    FileStream streamFile = (FileStream)state[1];

        //    // End the operation.
        //    sourceBlob.EndDownloadToStream(result);

        //    // Close the stream.
        //    streamFile.Close();

        //    endDownload = DateTime.Now;
        //    SetText("Upload Complete \r\n" + (endUpload - startUpload).TotalSeconds.ToString() + " seconds");
        //}


        // This tool was originally designed for cross account Azure copies.  
        // Providing very basic upload and download support.  Other tools do it better.
        // This will block the current thread and take a long time to complete.
        private void DownloadVHDFromSource(string DownloadFileName)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgSourceStore.CurrentRow;

            string sourceVHDName = singleRow.Cells["cSourceVHDName"].Value.ToString();
            
            txtMain.Text += "Starting Download of VHD.  This will take a while.\r\n";
            labelToDo.Text = "Starting Download of VHD.  This will take a while.\r\nPlease do not close this form or the download will stop.";
            startDownload = DateTime.Now;

            CloudStorageAccount sourceStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("SourceStorageConnection"));
            CloudBlobClient sourceBlobClient = sourceStorageAccount.CreateCloudBlobClient();

            // Retrieve reference to a previously created container
            CloudBlobContainer sourceContainer = sourceBlobClient.GetContainerReference(containerName);

            // Created the container if it does not exist
            CreateContainerIfNeeded(sourceContainer);

            // Get the VHD to download from the selected items
            CloudPageBlob sourceBlob = sourceContainer.GetPageBlobReference(sourceVHDName);

            // Set a new timeout for the huge download
            BlobRequestOptions setOptions = new BlobRequestOptions();
            setOptions.Timeout = new TimeSpan(10, 0, 0);

            // Download the blob to a file stream.
            FileStream streamFile = new FileStream(DownloadFileName, FileMode.Create);
            sourceBlob.BeginDownloadToStream(streamFile, null, setOptions, DownloadBlobToStreamCallback, new Object[] { sourceBlob, streamFile });

            lblStatus.Text = "Download Started \r\nDO NOT CLOSE FORM";
        }

        private void DownloadBlobToStreamCallback(IAsyncResult result)
        {
            // Get array passed to callback.
            Object[] state = (Object[])result.AsyncState;

            CloudPageBlob sourceBlob = (CloudPageBlob)state[0];
            FileStream streamFile = (FileStream)state[1];

            // End the operation.
            sourceBlob.EndDownloadToStream(result);

            // Close the stream.
            streamFile.Close();

            endDownload = DateTime.Now;
            SetText("Download Complete \r\n" + (endDownload - startDownload).TotalSeconds.ToString() + " seconds");
        }

        // Approach per http://stackoverflow.com/questions/10775367/cross-thread-operation-not-valid-control-textbox1-accessed-from-a-thread-othe
        delegate void SetStatusText(string text);
        private void SetText(string text)
        {
            // InvokeRequired required compares the thread ID of the 
            // calling thread to the thread ID of the creating thread. 
            // If these threads are different, it returns true. 
            if (this.lblStatus.InvokeRequired)
            {
                SetStatusText d = new SetStatusText(SetText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.lblStatus.Text = text;
            }
        }

        // Method to load the data grid for Local VMs
        private void LoadLocalVM()
        {
            // Clear the existing items
            dgLocalVMList.Rows.Clear();

            // Load all items from the folder location
            if (Directory.Exists(localVMStorageLocation))
            {
                string[] vmFiles = Directory.GetFiles(localVMStorageLocation);
                foreach (string vmFile in vmFiles)
                    ProcessSingleVMFile(vmFile);
            }
        }

        // Method to process a VM File
        private void ProcessSingleVMFile(string singleVMFile)
        {
            if (Path.GetExtension(singleVMFile).ToUpper() == ".XML")
            {
                //// Using XML Document since the files are small
                //XmlDocument xDoc = new XmlDocument();
                //xDoc.Load(singleVMFile);
                //string vmVMName = xDoc.SelectSingleNode("/*[local-name()='PersistentVM' and namespace-uri()='']/*[local-name()='RoleName' and namespace-uri()='']").Value;
                //string vmSize = xDoc.SelectSingleNode("/*[local-name()='PersistentVM' and namespace-uri()='']/*[local-name()='RoleSize' and namespace-uri()='']").Value;

                string[] parseFileName = Path.GetFileNameWithoutExtension(singleVMFile).Split(new Char[] { '_' });

                // Look through all the files and only add items that can parse out the 4 items
                if (parseFileName.Length == 6)
                {
                    string vmServiceName = parseFileName[0];
                    string vmDeployment = parseFileName[1];
                    string vmVMName = parseFileName[2];
                    string vmLocation = parseFileName[3];
                    string vmSize = parseFileName[4];
                    string fileType = parseFileName[5];

                    string[] dataRow = { "false", vmServiceName, vmDeployment, vmVMName, vmSize, vmLocation, "", singleVMFile, singleVMFile.Replace("_VM.xml","_ROLE.xml") };

                    // Add the VM to the Data Grid if not a BackUp
                    if (fileType != "ROLE")
                    {
                        dgLocalVMList.Rows.Add(dataRow);
                    }
                }
            }
        }
    }
}
