﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.IO;
using System.Text;
using System.Windows.Forms;
using CopyAzureStorage;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using System.Threading;
using System.Xml;
using System.Diagnostics;

namespace CopyAzureStorage
{
    // This contains all the events for the form.
    public partial class VHDandVMHelper : Form
    {
        string selectedSourceDisk = string.Empty;
        string localVMStorageLocation = string.Empty;
        string vmExportServiceName = string.Empty;
        string vmExportName = string.Empty;
        string vmExportLocation = string.Empty;
        string vmExportSize = string.Empty;
        public string postBody = string.Empty;
        int numberToBringOnline = 0;
        int numberToTakeOffline = 0;
        int numberToBringOnlineError = 0;
        int numberToTakeOfflineError = 0;
        List<string> lockedServicesUploadVM = new List<string>();
        List<string> lockedServicesExportVM = new List<string>();

        public VHDandVMHelper()
        {
            InitializeComponent();

            // Try to read the advanced settings from the App.Config
            // If we get an Exception on the first one, that's ok because we'll assume False
            try
            {
                certificatePath = CloudConfigurationManager.GetSetting("CertificatePath");
                globalAzureSourceAccountID = CloudConfigurationManager.GetSetting("SourceSubscriptionID");
                hasRESTApiEnabled = Convert.ToBoolean(CloudConfigurationManager.GetSetting("EnableRESTApi"));
                if (hasRESTApiEnabled)
                    groupRestAPI.Enabled = true;
            }
            catch { }

            // UI Stuff to Disable and Enable the Destination Account details
            if (checkBoxSourceOnly.Checked)
                groupDestination.Enabled = false;
            else
                groupDestination.Enabled = true;

            try
            {
                CloudStorageAccount tryToParse = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("DestinationStorageConnection"));

                // If I get this far I know I have a Destination Connection 
                checkBoxSourceOnly.Checked = false;
            }
            catch { }
            
            try
            {
                localVMStorageLocation = CloudConfigurationManager.GetSetting("LocalVMLocation");
                LoadLocalVM();
            }
            catch { }
        }

        private void btnListContainers_Click(object sender, EventArgs e)
        {
            // Enable the Source Group
            groupSource.Enabled = true;

            UpdateSourceVHDListBox();
            UpdateDestinationVHDListBox();

            if (hasRESTApiEnabled)
            {
                UpdateRESTApiDetails();
            }

            labelToDo.Text = "Step 3: Select the VHDS to Copy.\r\nStep 4: Set the VHD Name in the format of *.vhd.  TAB out of the field.";
        }

        private void btnCopy_Click(object sender, EventArgs e)
        {
            txtMain.Text += "Copy in progress for VHD names " + txtNewVHDName.Text + " to the Destination location.\r\n";
            CopyVHD();

            // Give is a few seconds to complete then update the ListBox
            Thread.Sleep(3000);
            UpdateDestinationVHDListBox();
            txtMain.Text += "Start of copy complete.\r\n";
        }

        // Update or Remove this logic to copy something other than VHD's using this tool.
        private void txtNewVHDName_Validating(object sender, CancelEventArgs e)
        {
            string VHDSText = txtNewVHDName.Text.Trim();

            if (!VHDSText.EndsWith(".vhd") || checkBoxSourceOnly.Checked == true)
            {
                labelToDo.Text = "ERROR: Rename VHD To must be in the format of *.vhd.  Then TAB out of the field.";
                btnCopy.Enabled = false;
            }
            else
            {
                labelToDo.Text = "Step 4: Click Copy Selected VHD to Destination.";
                btnCopy.Enabled = true;
            }

        }

        private void btnBreakSource_Click(object sender, EventArgs e)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgSourceStore.CurrentRow;

            string vhdName = singleRow.Cells["cSourceVHDName"].Value.ToString();

            txtMain.Text += "Breaking Lease on VHD " + vhdName + "\r\n";

            CloudStorageAccount sourceStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("SourceStorageConnection"));

            // Create the blob client using the Accounts above
            CloudBlobClient sourceBlobClient = sourceStorageAccount.CreateCloudBlobClient();

            CloudBlobContainer sourceContainer = sourceBlobClient.GetContainerReference(containerName);
            CloudBlob sourceBlob = sourceContainer.GetBlobReference(vhdName);

            TimeSpan breakTime = new TimeSpan(0, 0, 1);

            sourceBlob.BreakLease(breakTime);

            txtMain.Text += "Source VHD Breaking complete.\r\n";

            // Give is a few seconds to complete then update the ListBox
            Thread.Sleep(2000);
            UpdateSourceVHDListBox();
        }

        private void btnBreakDestination_Click(object sender, EventArgs e)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgDestinationStore.CurrentRow;

            string vhdName = singleRow.Cells["cDestinationVHDName"].Value.ToString();

            txtMain.Text += "Breaking Lease on VHD " + vhdName + "\r\n";

            CloudStorageAccount destinationStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("DestinationStorageConnection"));

            // Create the blob client using the Accounts above
            CloudBlobClient destinationBlobClient = destinationStorageAccount.CreateCloudBlobClient();

            CloudBlobContainer destinationContainer = destinationBlobClient.GetContainerReference(containerName);
            CloudBlob destinationBlob = destinationContainer.GetBlobReference(vhdName);

            TimeSpan breakTime = new TimeSpan(0, 0, 1);

            txtMain.Text += "Lease Status is " + destinationBlob.Properties.LeaseStatus.ToString() + "\r\n";

            destinationBlob.BreakLease(breakTime);

            txtMain.Text += "Destination VHD Breaking complete.\r\n";

            // Give is a few seconds to complete then update the ListBox
            Thread.Sleep(2000);
            UpdateDestinationVHDListBox();
        }

        private void btnDeleteSource_Click(object sender, EventArgs e)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgSourceStore.CurrentRow;

            string vhdName = singleRow.Cells["cSourceVHDName"].Value.ToString();

            txtMain.Text += "Deleting Source VHD " + vhdName + "\r\n";
            labelToDo.Text = "Deleting Source VHD " + vhdName + "\r\n";

            CloudStorageAccount sourceStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("SourceStorageConnection"));

            // Create the blob client using the Accounts above
            CloudBlobClient sourceBlobClient = sourceStorageAccount.CreateCloudBlobClient();

            CloudBlobContainer sourceContainer = sourceBlobClient.GetContainerReference(containerName);
            CloudBlob sourceBlob = sourceContainer.GetBlobReference(vhdName);

            sourceBlob.Delete();

            txtMain.Text += "Source VHD Deleted.\r\n";

            // Give is a few seconds to complete then update the ListBox
            Thread.Sleep(2000);
            UpdateSourceVHDListBox();

            txtMain.Text += "Deleting " + vhdName + " Complete\r\n";
            labelToDo.Text = "Deleting " + vhdName + " Complete\r\n";
        }

        private void btnDeleteDestination_Click(object sender, EventArgs e)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgDestinationStore.CurrentRow;

            string vhdName = singleRow.Cells["cDestinationVHDName"].Value.ToString();

            txtMain.Text += "Deleting Destination VHD " + vhdName + "\r\n";
            labelToDo.Text = "Deleting Destination VHD " + vhdName + "\r\n";

            CloudStorageAccount destinationStorageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("destinationStorageConnection"));

            // Create the blob client using the Accounts above
            CloudBlobClient destinationBlobClient = destinationStorageAccount.CreateCloudBlobClient();

            CloudBlobContainer destinationContainer = destinationBlobClient.GetContainerReference(containerName);
            CloudBlob sourceBlob = destinationContainer.GetBlobReference(vhdName);

            sourceBlob.Delete();

            // Give is a few seconds to complete then update the ListBox
            Thread.Sleep(2000);
            UpdateDestinationVHDListBox();

            txtMain.Text += "Deleting " + vhdName + " Complete\r\n";
            labelToDo.Text = "Deleting " + vhdName + " Complete\r\n";
        }

        private void btnCopyStatus_Click(object sender, EventArgs e)
        {
            //if (checkCopyStatus.CopyState == null)
            //{
            //    txtMain.Text += "Copy status is NULL.\r\n";
            //    labelToDo.Text = "Copy status is NULL. \r\nIf this is cross region copy it is likely still in progress.";
            //}
            //else
            //{
            //    if (checkCopyStatus.CopyState.Status == CopyStatus.Success)
            //    {
            //        txtMain.Text += "Copy complete.\r\n";
            //        labelToDo.Text = "COPY COMPLETE. \r\nClose tool and update connection strings to run again.";
            //    }
            //    else
            //    {
            //        txtMain.Text += "Copy Status is " + checkCopyStatus.CopyState.Status.ToString() + ".\r\n";
            //        labelToDo.Text = "Copy Status is " + checkCopyStatus.CopyState.Status.ToString() + "";
            //    }
            //}
        }

        // UI Stuff to Disable and Enable the Destination Account details
        private void checkBoxSourceOnly_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxSourceOnly.Checked)
            {
                groupDestination.Enabled = false;
            }
            else
            {
                groupDestination.Enabled = true;
            }
        }

        private void btnUploadVHD_Click(object sender, EventArgs e)
        {
            // Saving this for next release.

            //lblStatus.Enabled = true;

            //DialogResult selectFile = openFileDialogVHD.ShowDialog();

            //if (selectFile == DialogResult.OK)
            //{
            //    UploadVHDToSource(openFileDialogVHD.FileName);
            //}

        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            lblStatus.Enabled = true;

            DialogResult selectFile = saveFileDialogVHD.ShowDialog();

            if (selectFile == DialogResult.OK)
            {
                DownloadVHDFromSource(saveFileDialogVHD.FileName);
            }
        }

        private void btnVMDetails_Click(object sender, EventArgs e)
        {
            // Do nothing if no rows and nothing selected
            if (dgVMList.Rows.Count > 0)
            {
                if (dgVMList.CurrentRow.Selected)
                {
                    // Get the selected row
                    DataGridViewRow singleRow = dgVMList.CurrentRow;

                    //string vmName = singleRow.Cells["cVMName"].Value.ToString();
                    string vmName = singleRow.Cells["cVMServiceName"].Value.ToString();

                    string createURL = "https://management.core.windows.net/" + globalAzureSourceAccountID + "/services/hostedservices/" + vmName + "/deploymentslots/Production";
                    GetRestAPIInfo(createURL, "GET", "getvmdetails");
                }
            }
        }

        private void btnCreateDisk_Click(object sender, EventArgs e)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgSourceStore.CurrentRow;

            string sourceVHDName = singleRow.Cells["cSourceVHDName"].Value.ToString();

            CreateDisk createDisk = new CreateDisk();
            createDisk.selectedSourceDisk = sourceVHDName;
            createDisk.FormClosing += new FormClosingEventHandler(createDisk_FormClosing);
            createDisk.Visible = true;
        }

        void createDisk_FormClosing(object sender, FormClosingEventArgs e)
        {
            CreateDisk createDisk = (CreateDisk)sender;
            labelToDo.Text = createDisk.selectedHasOS;
        }

        private void btnImageDetails_Click(object sender, EventArgs e)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgImages.CurrentRow;

            // Find the Row in the existing Disk Collection
            int selectedDisk = 0;

            // Must be a better way to look for an item - just didn't want to spend time looking for it
            foreach (ImagesOSImage item in sourceImages.Items)
            {
                // If this name matches exit the for each
                if (item.Name == singleRow.Cells["cImageName"].Value.ToString())
                {
                    break;
                }

                // Not found so add one to count
                selectedDisk++;
            }

            StringBuilder btnImageDetails = new StringBuilder();
            btnImageDetails.Append("Name: " + sourceImages.Items[selectedDisk].Name);
            btnImageDetails.Append("\r\nLocation: " + sourceImages.Items[selectedDisk].Location);
            btnImageDetails.Append("\r\nSize in GB: " + sourceImages.Items[selectedDisk].LogicalSizeInGB);
            btnImageDetails.Append("\r\nCategory: " + sourceImages.Items[selectedDisk].Category);
            btnImageDetails.Append("\r\nLabel: " + sourceImages.Items[selectedDisk].Label);
            btnImageDetails.Append("\r\nOperating System: " + sourceImages.Items[selectedDisk].OS);
            btnImageDetails.Append("\r\nMedia Link: " + sourceImages.Items[selectedDisk].MediaLink);

            MessageBox.Show(btnImageDetails.ToString(), "Details for Custom Image " + sourceImages.Items[selectedDisk].Name);

        }

        private void btnDiskDetails_Click(object sender, EventArgs e)
        {
            // Get the selected row
            DataGridViewRow singleRow = dgAllDisks.CurrentRow;

            // Find the Row in the existing Disk Collection
            int selectedDisk = 0;

            // Must be a better way to look for an item - just didn't want to spend time looking for it
            foreach (DisksDisk item in sourceDisks.Items)
            {
                // If this name matches exit the for each
                if (item.Name == singleRow.Cells["cDiskName"].Value.ToString())
                {
                    break;
                }

                // Not found so add one to count
                selectedDisk++;
            }

            string hostedServiceName = "None";
            string virtualMachineName = "None";

            // Getting items that might not exist
            if (sourceDisks.Items[selectedDisk].AttachedTo != null)
            {
                hostedServiceName = sourceDisks.Items[selectedDisk].AttachedTo[0].HostedServiceName;
                virtualMachineName = sourceDisks.Items[selectedDisk].AttachedTo[0].RoleName;
            }

            StringBuilder btnDiskDetails = new StringBuilder();
            btnDiskDetails.Append("Using Service: " + hostedServiceName);
            btnDiskDetails.Append("\r\nAttached to Virtual Machine: " + virtualMachineName);
            btnDiskDetails.Append("\r\nName: " + sourceDisks.Items[selectedDisk].Name);
            btnDiskDetails.Append("\r\nLocation: " + sourceDisks.Items[selectedDisk].Location);
            btnDiskDetails.Append("\r\nSize in GB: " + sourceDisks.Items[selectedDisk].LogicalDiskSizeInGB);
            btnDiskDetails.Append("\r\nLabel: " + sourceDisks.Items[selectedDisk].Label);
            btnDiskDetails.Append("\r\nIs Corrupted: " + sourceDisks.Items[selectedDisk].IsCorrupted);
            btnDiskDetails.Append("\r\nHas Operating System: " + sourceDisks.Items[selectedDisk].HasOperatingSystem);
            btnDiskDetails.Append("\r\nOperating System: " + sourceDisks.Items[selectedDisk].OS);
            btnDiskDetails.Append("\r\nMedia Link: " + sourceDisks.Items[selectedDisk].MediaLink);
            btnDiskDetails.Append("\r\nSource Image: " + sourceDisks.Items[selectedDisk].SourceImageName);

            MessageBox.Show(btnDiskDetails.ToString(), "Details for Disk " + sourceDisks.Items[selectedDisk].Name);
        }

        private void dgSourceStore_SelectionChanged(object sender, EventArgs e)
        {
            btnDownload.Enabled = true;

            // Get the selected row
            DataGridViewRow singleRow = dgSourceStore.CurrentRow;

            string sourceVHDName = singleRow.Cells["cSourceVHDName"].Value.ToString();
            string sourceVHDLeased = singleRow.Cells["cSourceVHDLease"].Value.ToString();

            if (sourceVHDLeased.ToUpper().Contains("LEASED") || checkBoxSourceOnly.Checked == true)
            {
                if (checkBoxSourceOnly.Checked == true)
                    labelToDo.Text = "To Copy a VHD set Destination and uncheck the Source Only Flag.\r\nYou can only Download a VHD.";
                else
                    labelToDo.Text = "You can not copy a Leased VHD.  \r\nUpload or Download VHD or select another VHD to copy.";

                btnCopy.Enabled = false;
                btnCreateDisk.Enabled = false;
            }
            else
            {
                // Enable the ability to create a Azure Disk if the Disk isn't leased
                btnCreateDisk.Enabled = true;

                labelToDo.Text = "Step 4: Set the VHD Name in the format of *.vhd.  TAB out of the field.";
                txtNewVHDName.Enabled = true;
                txtNewVHDName.Text = sourceVHDName;
                if (!checkBoxSourceOnly.Checked)
                    btnCopy.Enabled = true;
            }
        }

        // Method to take all the selected VM's offline 
        private void btnVMTakeOffline_Click(object sender, EventArgs e)
        {
            // Reset the lock list
            lockedServicesExportVM.Clear();

            // Reset the counter just in case
            numberToTakeOffline = 0;
            numberToTakeOfflineError = 0;

            // Disable the button
            btnVMTakeOffline.Enabled = false;
            
            // Count the number of items to take offline
            foreach (DataGridViewRow singleRow in dgVMList.Rows)
            {
                if (singleRow.Cells["cVMTakeOffline"].Value.ToString() == "true")
                {
                    numberToTakeOffline++;
                }
            }

            // Create the Array
            BackgroundWorker[] virturalMachineArray = new BackgroundWorker[numberToTakeOffline];

            // Initialize BackgroudWorks Array
            for (int w = 0; w < numberToTakeOffline; w++)
            {
                virturalMachineArray[w] = new BackgroundWorker();
                virturalMachineArray[w].DoWork += new DoWorkEventHandler(backworkVMOffline_DoWork);
                virturalMachineArray[w].RunWorkerCompleted += new RunWorkerCompletedEventHandler(backworkVMOffline_RunWorkerCompleted);
                virturalMachineArray[w].ProgressChanged += new ProgressChangedEventHandler(backworkVMOffline_ProgressChanged);
                virturalMachineArray[w].WorkerReportsProgress = true;
                virturalMachineArray[w].WorkerSupportsCancellation = true;
            }

            // Loop Count
            int i = 0;

            foreach (DataGridViewRow singleRow in dgVMList.Rows)
            {
                if (singleRow.Cells["cVMTakeOffline"].Value.ToString() == "true")
                {
                    RestAPICallRequest singleRestCall = new RestAPICallRequest();

                    // Get the VM Details
                    singleRestCall.SubscriptionID = globalAzureSourceAccountID;
                    singleRestCall.RowPosition = singleRow.Index;
                    singleRestCall.ServiceName = singleRow.Cells["cVMServiceName"].Value.ToString();
                    singleRestCall.VMName = singleRow.Cells["cVMName"].Value.ToString();
                    singleRestCall.Location = singleRow.Cells["cVMLocation"].Value.ToString();
                    singleRestCall.Deployment = singleRow.Cells["cVMDeployment"].Value.ToString();

                    virturalMachineArray[i].RunWorkerAsync(singleRestCall);
                    i++;
                }
            }
        }

        // Method to put all the selected VM's back online
        private void btnVMRestore_Click(object sender, EventArgs e)
        {
            // Reset the lock list
            lockedServicesUploadVM.Clear();
            
            // Reset the counter just in case
            numberToBringOnline = 0;
            numberToBringOnlineError = 0;

            // Disable the button
            btnVMRestore.Enabled = false;
            
            // Count the number of items to take offline
            foreach (DataGridViewRow singleRow in dgLocalVMList.Rows)
            {
                if (singleRow.Cells["cLVMBringOnline"].Value.ToString() == "true")
                {
                    numberToBringOnline++;
                }
            }

            // Create the Array
            BackgroundWorker[] virturalMachineArray = new BackgroundWorker[numberToBringOnline];

            // Initialize BackgroudWorks Array
            for (int w = 0; w < numberToBringOnline; w++)
            {
                virturalMachineArray[w] = new BackgroundWorker();
                virturalMachineArray[w].DoWork += new DoWorkEventHandler(backworkVMOnline_DoWork);
                virturalMachineArray[w].RunWorkerCompleted += new RunWorkerCompletedEventHandler(backworkVMOnline_RunWorkerCompleted);
                virturalMachineArray[w].ProgressChanged += new ProgressChangedEventHandler(backworkVMOnline_ProgressChanged);
                virturalMachineArray[w].WorkerReportsProgress = true;
                virturalMachineArray[w].WorkerSupportsCancellation = true;
            }

            // Loop Count
            int i = 0;

            foreach (DataGridViewRow singleRow in dgLocalVMList.Rows)
            {
                if (singleRow.Cells["cLVMBringOnline"].Value.ToString() == "true")
                {
                    RestAPICallRequest singleRestCall = new RestAPICallRequest();

                    // Get the VM Details
                    singleRestCall.SubscriptionID = globalAzureSourceAccountID;
                    singleRestCall.RowPosition = singleRow.Index;
                    singleRestCall.ServiceName = singleRow.Cells["cLVMServiceName"].Value.ToString();
                    singleRestCall.VMName = singleRow.Cells["cLVMName"].Value.ToString();
                    singleRestCall.Deployment = singleRow.Cells["cLVMDeployment"].Value.ToString();
                    singleRestCall.FileLocationVM = singleRow.Cells["cVMLFileNameVM"].Value.ToString();
                    singleRestCall.FileLocationRole = singleRow.Cells["cVMLFileNameRole"].Value.ToString();

                    virturalMachineArray[i].RunWorkerAsync(singleRestCall);
                    i++;
                }
            }
        }

        private void backworkVMOffline_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;
            RestAPICallRequest singleRestCall = e.Argument as RestAPICallRequest;
            RestAPICallResponse response = new RestAPICallResponse();

            // Wait for no other operations to be running on this service
            while (true)
            {
                // Check if this service is already locked, if not lock it
                if (lockedServicesExportVM.Contains(singleRestCall.ServiceName))
                {
                    singleRestCall.Status = "Service in use.  Sleeping 30.";
                    worker.ReportProgress(0, singleRestCall);
                    Thread.Sleep(30000);
                }
                else
                {
                    lockedServicesExportVM.Add(singleRestCall.ServiceName);
                    break;
                }
            }
                                   
            // Set the Subscription ID
            singleRestCall.SubscriptionID = globalAzureSourceAccountID;
            singleRestCall.APICertificatePath = certificatePath;
            singleRestCall.HasError = false;

            // Create Object for the REST API
            RestAPI objRestAPI = new RestAPI();

            singleRestCall.Status = "Getting VM Details.";
            worker.ReportProgress(0, singleRestCall);

            // Get the VM Details
            singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deploymentslots/Production";
            singleRestCall.APIMethod = "GETVMDETAILSEXPORT";
            singleRestCall.APIOperation = "GET";
            singleRestCall.VMStoragePath = localVMStorageLocation;
            response = objRestAPI.ProcessRestAPI(singleRestCall);

            // Set the VM Role Count
            int vmRoleCount = response.VMRoleCount;

            if (!response.HasError)
            {
                singleRestCall.Status = "Stopping.";
                worker.ReportProgress(0, singleRestCall);

                // Stop the VM            
                singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deployments/" + singleRestCall.Deployment + "/roleinstances/" + singleRestCall.VMName + "/Operations";
                singleRestCall.APIMethod = "";
                singleRestCall.APIOperation = "POST";
                singleRestCall.APIPostText = "<ShutdownRoleOperation xmlns='http://schemas.microsoft.com/windowsazure' xmlns:i='http://www.w3.org/2001/XMLSchema-instance'><OperationType>ShutdownRoleOperation</OperationType></ShutdownRoleOperation>";
                response = objRestAPI.ProcessRestAPI(singleRestCall);

                if (!response.HasError)
                {
                    singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/operations/" + response.ASyncId;
                    singleRestCall.APIMethod = "";

                    // To get into the first loop
                    response.Status = "INPROGRESS";

                    int countStop = 1;

                    while (response.Status.ToUpper() == "INPROGRESS")
                    {
                        Thread.Sleep(5000);
                        singleRestCall.APIOperation = "GET";
                        response = objRestAPI.ProcessRestAPI(singleRestCall);

                        if (response.HasError)
                            break;

                        singleRestCall.Status = "Stopping: " + (countStop * 5).ToString() + " seconds.";
                        worker.ReportProgress(0, singleRestCall);
                        countStop++;
                    }

                    //if (false)
                    if (!response.HasError)
                    {
                        singleRestCall.Status = "Deleting VM.";
                        worker.ReportProgress(0, singleRestCall);

                        // If no error, Delete the VM
                        // If only one Role use basic method
                        if (vmRoleCount == 1)
                        {
                            singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deploymentslots/Production";
                        }
                        else
                        {
                            // If more than one Role just remove this role
                            singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deployments/" + singleRestCall.Deployment + "/roles/" + singleRestCall.VMName;
                        }

                        singleRestCall.APIMethod = "";
                        singleRestCall.APIOperation = "DELETE";
                        response = objRestAPI.ProcessRestAPI(singleRestCall);

                        singleRestCall.Status = "Deleting. Getting Status.";
                        worker.ReportProgress(0, singleRestCall);

                        if (!response.HasError)
                        {
                            singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/operations/" + response.ASyncId;
                            singleRestCall.APIMethod = "";

                            // To get into the first loop
                            response.Status = "INPROGRESS";

                            countStop = 1;

                            while (response.Status.ToUpper() == "INPROGRESS")
                            {
                                Thread.Sleep(5000);
                                singleRestCall.APIOperation = "GET";
                                response = objRestAPI.ProcessRestAPI(singleRestCall);

                                if (response.HasError)
                                    break;

                                singleRestCall.Status = "Deleting: " + (countStop * 5).ToString() + " seconds.";
                                worker.ReportProgress(0, singleRestCall);
                                countStop++;
                            }

                            singleRestCall.Status = "Complete.";
                            worker.ReportProgress(0, singleRestCall);
                        }
                    }
                }
            }

            if (response.HasError)
            {
                singleRestCall.HasError = true; 
                singleRestCall.Status = "Error. URL - " + response.ErrorURL + " Operation - " + response.ErrorOperation + " Text - "+ response.ErrorText;
                worker.ReportProgress(0, singleRestCall);
            }

            // Unlock the service
            lockedServicesExportVM.Remove(singleRestCall.ServiceName);

            e.Result = singleRestCall;
        }

        private void backworkVMOnline_DoWork(object sender, DoWorkEventArgs e)
        {           
            BackgroundWorker worker = sender as BackgroundWorker;
            RestAPICallRequest singleRestCall = e.Argument as RestAPICallRequest;
            RestAPICallResponse response = new RestAPICallResponse();

            while (true)
            {
                // Check if this service is already locked, if not lock it
                if (lockedServicesUploadVM.Contains(singleRestCall.ServiceName))
                {
                    singleRestCall.Status = "Service in use.  Sleeping 30.";
                    worker.ReportProgress(0, singleRestCall);
                    Thread.Sleep(30000);
                }
                else
                {
                    lockedServicesUploadVM.Add(singleRestCall.ServiceName);
                    break;
                }
            }
            
            // Set the Subscription ID
            singleRestCall.SubscriptionID = globalAzureSourceAccountID;
            singleRestCall.APICertificatePath = certificatePath;
            singleRestCall.HasError = false;

            // Create Object for the REST API
            RestAPI objRestAPI = new RestAPI();

            singleRestCall.Status = "Getting VM Details.";
            worker.ReportProgress(0, singleRestCall);

            // Get the VM Details
            singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deploymentslots/Production";
            singleRestCall.APIMethod = "GETVMDETAILSIMPORT";
            singleRestCall.APIOperation = "GET";
            response = objRestAPI.ProcessRestAPI(singleRestCall);

            int vmRoleCount = response.VMRoleCount;

            if (!response.HasError)
            {
                singleRestCall.Status = "Loading VM.";
                worker.ReportProgress(0, singleRestCall);

                XmlDocument oDoc = new XmlDocument();

                // Decide if this is going to add a new Role or if Roles already exist to add to
                if (vmRoleCount == 0)
                {
                    oDoc.Load(singleRestCall.FileLocationVM);
                    singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deployments";
                }
                else
                {
                    oDoc.Load(singleRestCall.FileLocationRole);
                    singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deployments/" + singleRestCall.Deployment + "/roles";
                }

                singleRestCall.APIMethod = "";
                singleRestCall.APIOperation = "POST";
                singleRestCall.APIPostText = oDoc.OuterXml;

                response = objRestAPI.ProcessRestAPI(singleRestCall);

                if (!response.HasError)
                {
                    singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/operations/" + response.ASyncId;
                    singleRestCall.APIMethod = "";
                    singleRestCall.APIPostText = "";

                    // To get into the first loop
                    response.Status = "INPROGRESS";

                    int countStop = 1;

                    while (response.Status.ToUpper() == "INPROGRESS")
                    {
                        Thread.Sleep(5000);
                        singleRestCall.APIOperation = "GET";
                        response = objRestAPI.ProcessRestAPI(singleRestCall);

                        // If Error, break
                        if (response.HasError)
                            break;

                        singleRestCall.Status = "Creating VM: " + (countStop * 5).ToString() + " seconds.";
                        worker.ReportProgress(0, singleRestCall);
                        countStop++;
                    }

                    if (!response.HasError)
                    {
                        // Starting the VM
                        singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/services/hostedservices/" + singleRestCall.ServiceName + "/deployments/" + singleRestCall.Deployment + "/roleinstances/" + singleRestCall.VMName + "/Operations";
                        singleRestCall.APIMethod = "";
                        singleRestCall.APIOperation = "POST";
                        singleRestCall.APIPostText = "<StartRoleOperation xmlns='http://schemas.microsoft.com/windowsazure' xmlns:i='http://www.w3.org/2001/XMLSchema-instance'><OperationType>StartRoleOperation</OperationType></StartRoleOperation>";
                        response = objRestAPI.ProcessRestAPI(singleRestCall);

                        if (!response.HasError)
                        {
                            singleRestCall.Status = "Starting VM.";
                            worker.ReportProgress(0, singleRestCall);

                            // Check Start Status
                            singleRestCall.APIURL = "https://management.core.windows.net/" + singleRestCall.SubscriptionID + "/operations/" + response.ASyncId;
                            singleRestCall.APIOperation = "GET";
                            singleRestCall.APIPostText = "";
                            response = objRestAPI.ProcessRestAPI(singleRestCall);

                            response.Status = "INPROGRESS";

                            countStop = 1;

                            while (response.Status.ToUpper() == "INPROGRESS")
                            {
                                Thread.Sleep(5000);
                                singleRestCall.APIOperation = "GET";
                                response = objRestAPI.ProcessRestAPI(singleRestCall);

                                if (response.HasError)
                                    break;

                                singleRestCall.Status = "Starting VM: " + (countStop * 5).ToString() + " seconds.";
                                worker.ReportProgress(0, singleRestCall);
                                countStop++;
                            }

                            singleRestCall.Status = "Complete.";
                            worker.ReportProgress(0, singleRestCall);
                        }
                    }
                }
            }

            if (response.HasError)
            {
                singleRestCall.HasError = true;
                singleRestCall.Status = "Error. URL - " + response.ErrorURL + " Operation - " + response.ErrorOperation + " Text - " + response.ErrorText;
                worker.ReportProgress(0, singleRestCall);
            }

            // Unlock the service
            lockedServicesUploadVM.Remove(singleRestCall.ServiceName);

            e.Result = singleRestCall;
        }

        private void backworkVMOffline_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            RestAPICallRequest singleRestCall = e.UserState as RestAPICallRequest;
            DataGridViewRow singleRow = dgVMList.Rows[singleRestCall.RowPosition];
            singleRow.Cells["cVMStatus"].Value = singleRestCall.Status;
        }

        private void backworkVMOnline_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            RestAPICallRequest singleRestCall = e.UserState as RestAPICallRequest;
            DataGridViewRow singleRow = dgLocalVMList.Rows[singleRestCall.RowPosition];
            singleRow.Cells["cLVMStatus"].Value = singleRestCall.Status;
        }

        private void backworkVMOffline_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            RestAPICallRequest singleRestCall = e.Result as RestAPICallRequest;
            DataGridViewRow singleRow = dgVMList.Rows[singleRestCall.RowPosition];

            singleRow.Cells["cVMTakeOffline"].Value = "false";

            if (singleRestCall.HasError)
            {
                numberToTakeOfflineError++;
            }
            else
            {
                singleRow.Visible = false;
                numberToTakeOffline--;
            }
            
            // This means all the treads have completed.  If any have an error do not refresh.
            if (numberToTakeOffline == 0 && numberToTakeOfflineError == 0)
            {
                // Reload the local files
                LoadLocalVM();
                // Refresh the Online VM
                UpdateRESTApiDetails();
            }

            // This means we are all done
            if(numberToTakeOffline - numberToTakeOfflineError == 0)
                btnVMTakeOffline.Enabled = true;
        }

        private void backworkVMOnline_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            RestAPICallRequest singleRestCall = e.Result as RestAPICallRequest;
            DataGridViewRow singleRow = dgLocalVMList.Rows[singleRestCall.RowPosition];

            singleRow.Cells["cLVMBringOnline"].Value = "false";

            if (singleRestCall.HasError)
            {
                numberToBringOnlineError++;
            }
            else
            {
                singleRow.Visible = false;
                File.Delete(singleRow.Cells["cVMLFileNameRole"].Value.ToString());
                File.Delete(singleRow.Cells["cVMLFileNameVM"].Value.ToString());
                numberToBringOnline--;
            }
            
            // This means all the treads have completed.  If any have an error do not refresh.
            if (numberToBringOnline == 0 && numberToBringOnlineError == 0)
            {
                // Reload the local files
                LoadLocalVM();
                // Refresh the Online VM
                UpdateRESTApiDetails();
            }

            // This means we are all done
            if (numberToBringOnline - numberToBringOnlineError == 0)
                btnVMRestore.Enabled = true;
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ProcessStartInfo sInfo = new ProcessStartInfo("http://eepurl.com/u77Uz");
            Process.Start(sInfo);
        }

        private void linkLabel3_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ProcessStartInfo sInfo = new ProcessStartInfo("http://www.BizTalkGuru.com");
            Process.Start(sInfo);
        }

        private void linkLabel4_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ProcessStartInfo sInfo = new ProcessStartInfo("http://www.BizTalkGurus.com/p/azure_vmtool_known_issues.aspx");
            Process.Start(sInfo);
        }

        private void linkLabel2_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                ProcessStartInfo sInfo = new ProcessStartInfo("mailto:stephen.thomas@kendalltechnologies.com?subject=VM Tool Issue");
                Process.Start(sInfo);
                //System.Diagnostics.Process.Start("mailto:stephen.thomas@kendalltechnologies.com?subject=VM Tool Issue");
            }
            catch (Exception)
            {
                MessageBox.Show("Please email stephen.thomas@kendalltechnologies.com with any issues.", "No Default Mail Client Detected");
            } 
        }
    }
}
